package eCommerce;

import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
//import java.util.Iterator;
//import java.util.HashMap;

import javax.faces.bean.ManagedBean;
import javax.faces.context.FacesContext;

//this class manages the storefront and the shopping cart
@ManagedBean
public class OrderBuilder implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	//holds all products from db
	private ArrayList<Product> products;
	//holds shopping cart order
	private ArrayList<OrderedItems> order;

	//addresses of pages to return
	private String cartURL = "cart.jsf";
	private String loginplace = "signin.jsf";
	private String shopURL = "shopping.jsf";
	private String checkoutURL = "details.jsf";
	
	//for testing
	private String test = "HEYOH";
	private String debugMsg1 = "1";
	private String debugMsg2 = "2";
	private String debugMsg3 = "3";
	private String message = "";
	private String testStep = "ZERO";
	
	//which product is being purchased (set by action listener with add-to-cart button)
	private int buyProduct;
	
	//for shopping cart
	private BigDecimal taxrate = new BigDecimal("0.15"); //15%
	private BigDecimal subTotal = new BigDecimal("0");
	private BigDecimal tax = new BigDecimal("0");
	private BigDecimal total = new BigDecimal("0");
	
	//constructor - draws products from db
		public OrderBuilder(){
			
			this.test = "HELLO1!";
			try {
				this.test = "HELLO2!";
				DataAccess myAccess = new DataAccess();
				products = myAccess.getProductArrayList();
			} catch (SQLException e) {
				e.printStackTrace();
			}
			order = new ArrayList<OrderedItems>();
		}
	
	//getters and setters
	public BigDecimal getTaxrate() {
		return taxrate;
	}
	public String getTaxPercent() {
		BigDecimal hundred = new BigDecimal("100");
		return taxrate.multiply(hundred).stripTrailingZeros().toPlainString();
	}
	public BigDecimal getSubTotal() {
		return subTotal.setScale(2, BigDecimal.ROUND_HALF_UP);
	}
	public BigDecimal getTotal() {
		return total.setScale(2, BigDecimal.ROUND_HALF_UP);
	}
	public BigDecimal getTax() {
		return tax.setScale(2, BigDecimal.ROUND_HALF_UP);
	}
	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}
	
	public String getShopURL() {
		return shopURL;
	}

	public void setShopURL(String shopURL) {
		this.shopURL = shopURL;
	}
	
	public int getBuyProduct() {
		return buyProduct;
	}

	public void setBuyProduct(int buyProduct) {
		this.buyProduct = buyProduct;
	}

	public ArrayList<Product> getProducts(){
		return products;
	}
	
	public ArrayList<OrderedItems> getOrder(){
		return order;
	}
	
	public String getTest(){
		return test;
	}
	
	public String getCartURL(){
		return cartURL;
	}
	
	//testing getters and setters
	public String getDebugMsg1() {
		//debugMsg1 = order.get(0).getDebugMsg1();
		return debugMsg1;
	}
	public void setDebugMsg1(String debugMsg1) {
		this.debugMsg1 = debugMsg1;
	}
	public String getDebugMsg2() {
		//debugMsg2 = order.get(0).getDebugMsg2();
		return debugMsg2;
	}
	public void setDebugMsg2(String debugMsg2) {
		this.debugMsg2 = debugMsg2;
	}
	public String getDebugMsg3() {
		//debugMsg3 = order.get(0).getDebugMsg3();
		return debugMsg3;
	}
	public void setDebugMsg3(String debugMsg3) {
		this.debugMsg3 = debugMsg3;
	}
	
	public String getTestStep() {
		return testStep;
	}
	public void setTestStep(String testStep) {
		this.testStep = testStep;
	}
		
	public String getLoginplace() {
		return loginplace;
	}
	public void setLoginplace(String loginplace) {
		this.loginplace = loginplace;
	}
	
	//add item to cart
	public String addToCart(){
		buyMe();
		updateTotal();
		return cartURL;
	}
	
	//finalize cart and place order
	public String checkout(){
		//do checkout stuff
		message = "";
		if (order.isEmpty())
		{
		   message = "NO SALE!";
		   return cartURL;
		}
		return checkoutURL;//go build/sign-in a customer for the order
	}
	
	//return to shopping screen
	public String goShop(){
		//nuthin'
		message = "";
		return shopURL;
	}
	
	//called when user wants to maintain session after an order
	public String goShopFin(){
		//nuthin'
		message = "";
		order.clear();//clear cart
		return shopURL;
	}
	
	//clear session after an order
	//http://stackoverflow.com/a/5620582
	public String goShopNew(){
		FacesContext.getCurrentInstance().getExternalContext().invalidateSession();
		return shopURL;
	}
	
	//handles update button in shopping cart
	public String updateCart(){
		message = "";
		int removeindex = -1;
		boolean remove;
		boolean done = false;
		//if any items are quantity = 0, remove them
		while (!done){
			remove = false;
			for (int i=0;i<order.size();i++){
				if (order.get(i).getQty() == 0 || order.get(i).isRemoveMe()){			
				   removeindex = i;
				   remove = true;
				}
		}
		if (remove) order.remove(removeindex);
		else done = true;
		}
		updateTotal();
		return cartURL;
	}
	
	//adds specific product to cart
	public void buyMe(){
		int newquantity = 1;
		BigDecimal newprice = new BigDecimal(products.get(buyProduct - 1).getPrice());
		int removeindex = -1;
		boolean remove = false;
		
		for (int i=0;i<order.size();i++){
			if (order.get(i).getProductID() == buyProduct){			
			   //if it's already in there, get its quantity and current price
			   newquantity = 1 + order.get(i).getQty();
			   newprice = newprice.add(order.get(i).getPrice());
			   removeindex = i;
			   remove = true;
			}
		}
		
		if (remove) order.remove(removeindex);
		
		//make a new item record at the end of the ArrayList
		OrderedItems item = new OrderedItems(
				products.get(buyProduct - 1).getProductId(),
				products.get(buyProduct - 1).getName(),
				products.get(buyProduct - 1).getImageSource(),
	            newprice.toPlainString(),
	            products.get(buyProduct - 1).getPrice(),
	            newquantity
				);
		order.add(item);
		
		//show latest item at the top
		Collections.reverse(order);
	}
	
	//update total/subtotal
	public void updateTotal(){
		subTotal = new BigDecimal("0");
		for (int i=0;i<order.size();i++)
		{
		subTotal = subTotal.add(order.get(i).getPrice());
		}
		tax = subTotal.multiply(taxrate);
		total = subTotal.add(tax);
    }
	

}
